using System;
using System.Configuration;
using System.Windows.Forms;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Security;
using DevExpress.ExpressApp.Win;
using DevExpress.ExpressApp.Win.Core;
using Xafari.Security;

namespace Sample.Win
{
	internal static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		private static void Main()
		{
#if EASYTEST
			DevExpress.ExpressApp.Win.EasyTest.EasyTestRemotingRegistration.Register();
#endif
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
		   // EntityFactory.Enabled = false;
			EditModelPermission.AlwaysGranted = System.Diagnostics.Debugger.IsAttached;
			// TODO: use just WinApplication class
			var application = new SampleFormsApplication();
			XafariSecuritySystem.SyncWithModel = true;
			application.CustomHandleException += application_CustomHandleException;
			application.ConnectionString = ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
#if EASYTEST
			if(ConfigurationManager.ConnectionStrings["EasyTestConnectionString"] != null) {
				application.ConnectionString = ConfigurationManager.ConnectionStrings["EasyTestConnectionString"].ConnectionString;
			}
#endif
			string useOldTemplates = ConfigurationManager.AppSettings["UseOldTemplates"];
			application.UseOldTemplates = useOldTemplates == null || useOldTemplates.ToLower() != "false";

			
			var moduleAssemblies = !string.IsNullOrEmpty(ConfigurationManager.AppSettings["Modules"]) ? ConfigurationManager.AppSettings["Modules"].Split(';') : null;
			application.Setup("", application.ConnectionString, moduleAssemblies);
			//application.Setup();
  
			application.Start();
		}

		private static void application_CustomHandleException(object sender, CustomHandleExceptionEventArgs e)
		{
			var application = sender as XafApplication;
			if (e.Exception.InnerException != null)
			{
				var argEx = e.Exception.InnerException as ArgumentNullException;
				if (argEx != null)
				{
					Messaging.GetMessaging(application).Show(argEx.Message, "Error", MessageBoxButtons.OK,
						MessageBoxIcon.Error);
					e.Handled = true;
				}
			}
		}
	}
}
