﻿using System.ComponentModel;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Persistent.Base;
using DevExpress.Persistent.Base.General;
using DevExpress.Persistent.Validation;
using DevExpress.Xpo;
using Xafari.BC.Xpo;
using Xafari.Base;
using Xafari.BC.HierarchicalData;
using Xafari.BC.Numerators;
using Sample.Security;

namespace Sample.Classifiers
{
	[DefaultClassOptions]
	[RuleIsReferenced("Department.Delete", DefaultContexts.Delete, typeof(InheritedUser), "Department", MessageTemplateMustBeReferenced = "There are references from employees.", InvertResult = true)]
	[Custom("GroupName", "Classifiers")]
	public class Department : ClassifierItemBase, ITreeNode, IIndexNumberSupport
	{
		public Department(Session session) : base (session) {}
		
		[Association]
        [ChildrenProperty]
		public XPCollection<Department> Children
		{
			get
			{
			    // Не возращается только что созданный дочерний элемент
                //return this.GetCollection<Department>("Children");
                var member = base.ClassInfo.GetMember("Children");
			    return CreateCollection<Department>(member);
			}
		}

		private Department _parent = null;

		[Association]
        [ParentProperty]
		public Department Parent
		{
			get
			{
				CheckCorrectValue(_parent);
				return this.GetPropertyValue<Department>("Parent");
			}
			set
			{
				CheckCorrectValue(value);
				this.SetPropertyValue("Parent", value);
			}
		}

		void CheckCorrectValue(Department v)
		{
			if (v == null)
				return;
			var p = v;
			RaiseException(p);
			while (p != null && p != p.Parent)
			{
				RaiseException(p);
				p = p.Parent;
			}
		}

		void RaiseException(Department p)
		{
			if (p != this)
				return;

			var objectSpace = XPObjectSpace.FindObjectSpaceByObject(p);
			var vr = Validator.RuleSet.ValidateTarget(objectSpace, p, ContextIdentifier.Save);
			throw new ValidationException("Cannot select yourself as parent.", vr);
		}

		protected override void OnChanged(string propertyName, object oldValue, object newValue)
		{
			if (propertyName == "Parent" && oldValue != null)
			{
				var _oldParent = (Department)oldValue;
				_oldParent.Children.Remove(this);
			}
			base.OnChanged(propertyName, oldValue, newValue);
		}

		string ITreeNode.Name
		{
			get { return this.Name; }
		}

		ITreeNode ITreeNode.Parent
		{
			get { return this.Parent; }
		}

		IBindingList ITreeNode.Children
		{
			get { return this.Children; }
		}

	    public int IndexNumber
	    {
	        get { return GetPropertyValue<int>("IndexNumber"); }
            set { SetPropertyValue("IndexNumber", value); }
	    }
	}
}
