using DevExpress.Data.Filtering;
using DevExpress.ExpressApp.Security.Strategy;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Xpo;
using DevExpress.ExpressApp;
using DevExpress.Persistent.Validation;
using Xafari.Security;
using Xafari.BC;
using Sample.Classifiers;

namespace Sample.Security
{
	[MapInheritance(MapInheritanceType.ParentTable)]
	public class InheritedUser : SecuritySystemUser
	{
		public InheritedUser(Session session) : base(session) { }

        [RuleRequiredField("departmentReqField", DefaultContexts.Save, "Department should be specified.")]
		public Department Department
		{
			get { return this.GetPropertyValue<Department>("Department"); }
			set { this.SetPropertyValue("Department", value); }
		}

		public override void AfterConstruction()
		{
			base.AfterConstruction();
			if (Department != null) return;
			var objectSpace = XPObjectSpace.FindObjectSpaceByObject(this);
			var defaultDepartment = objectSpace.Xafari().FindObject<Department>(CriteriaOperator.Parse("Code=?", "00"), true);
			if (defaultDepartment == null)
			{
				defaultDepartment = objectSpace.CreateObject<Department>();
				defaultDepartment.Code = "00";
				defaultDepartment.Name = "Default Department";
			}
			Department = defaultDepartment;
		}
	}

}
