﻿using System;
using System.Linq;
using System.Text;
using System.ComponentModel;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.DC;
using System.Collections.Generic;
using DevExpress.ExpressApp.Model;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.Actions;
using DevExpress.ExpressApp.Updating;
using DevExpress.ExpressApp.Model.Core;
using DevExpress.ExpressApp.Model.DomainLogics;
using DevExpress.ExpressApp.Model.NodeGenerators;
using Xafari.BC;
using DevExpress.Persistent.BaseImpl;
using DevExpress.ExpressApp.Xpo;
using Sample.Localization;
using Xafari.Security;
using DevExpress.ExpressApp.Security;
using Sample.Security;
using DevExpress.ExpressApp.Security.ClientServer;
using Xafari;
using System.Threading;
using DevExpress.ExpressApp.Validation;
using Xafari.BC.Numerators;
using Xafari.BC.Settings;
using Xafari.BC.Services;
using Xafari.BC.Xpo;
using Xafari.Utils;
using Sample.Settings;

namespace Sample
{
    [ToolboxItemFilter("Xaf.Platform.Win")]
    // For more typical usage scenarios, be sure to check out https://documentation.devexpress.com/eXpressAppFramework/clsDevExpressExpressAppModuleBasetopic.aspx.
    public sealed partial class SampleModule : XafariModuleBase, IDataServerSecurityProvider
    {
        private readonly SecurityStrategyComplex securityStrategyComplex;
        private const string AppName="Sample";
        private XafariAuthentication authentication;
    
        public SampleModule()
        {
            InitializeComponent();
            BaseObject.OidInitializationMode = OidInitializationMode.AfterConstruction;
            authentication = new XafariAuthentication();

            authentication.AuthenticationActiveDirectory.CreateUserAutomatically = true;
            authentication.AuthenticationActiveDirectory.LogonParametersType = typeof(XafariAuthenticationLogonParameters);
            authentication.AuthenticationStandard.LogonParametersType = typeof(XafariAuthenticationLogonParameters);
            authentication.LogonParametersType = typeof(XafariAuthenticationLogonParameters);
            securityStrategyComplex = new ThreadSafeSecurityStrategyComplex(typeof(InheritedUser), typeof(InheritedRole), authentication);
        }

        public override void Setup(XafApplication application)
        {
            base.Setup(application);
            application.DelayedViewItemsInitialization = true;
            application.Security = securityStrategyComplex;

            application.ApplicationName = AppName;

            application.CreateCustomObjectSpaceProvider += (sender, e) => {
                var threadSafeStrategyProvider = new ThreadSafeSelectDataSecurityProvider((SecurityStrategyComplex)application.Security);
                e.ObjectSpaceProviders.Add(new SecuredObjectSpaceProvider(threadSafeStrategyProvider, e.ConnectionString, e.Connection));
                e.ObjectSpaceProviders.Add(new XafariNonPersistentObjectSpaceProvider(application.TypesInfo, null));
            };
            application.DelayedViewItemsInitialization = true;
            application.CustomizeLanguagesList += application_CustomizeLanguagesList;
            application.DatabaseVersionMismatch += application_DatabaseVersionMismatch;
            application.CreateCustomTemplate += Application_CreateCustomTemplate; ;
            application.LoggedOn += Application_LoggedOn;
            application.SetupComplete += Application_SetupComplete;
        }

        private void Application_SetupComplete(object sender, EventArgs e)
        {
            if (this.Application.Security != null && this.Application.Security is SecurityStrategy)
            {
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Category));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Customer));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Employee));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Product));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Shipper));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Supplier));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(Order));
                //((SecurityStrategy)Application.Security).AnonymousAllowedTypes.Add(typeof(OrderDetails));

            }
        }

        public override void Setup(ApplicationModulesManager moduleManager)
        {
            base.Setup(moduleManager);
            var validationModule = moduleManager.Modules.FindModule(typeof(ValidationModule)) as ValidationModule;
            if (validationModule != null)
                validationModule.CustomizeApplicationRuntimeRules += validationModule_CustomizeApplicationModelRules;
        }

        protected override void OnServicesSettingUp()
        {
            base.OnServicesSettingUp();
            NumeratorManager.Instance.InitializeStrategy = InitializeServiceStrategy.OnApplicationSetupComplete;
        }

        void Application_LoggedOn(object sender, LogonEventArgs e)
        {
            var application = sender as XafApplication;
            if (application == null) return;
            if (SettingsAccessor.Instance != null) return;
        }

        static void validationModule_CustomizeApplicationModelRules(object sender, CustomizeRulesEventArgs e)
        {
            e.Rules.AddRange(new RuleHelper().CreateRuleUnique<ClassifierItemBase>("Code").ToArray());
        }
        public override void ExtendModelInterfaces(ModelInterfaceExtenders extenders)
        {
            base.ExtendModelInterfaces(extenders);
            extenders.Add<IModelBCSettings, SampleSettings>();
        }


    
        string GetLicenseAttribute(string attributeName)
        {
            if (string.IsNullOrEmpty(attributeName))
                return null;

            var licSettings = LicensingSettings();
            if (licSettings == null) return "1"; //äëÿ äåìîïðèìåðà, îòñóòâèå ñåòòèíãîâ çíà÷èò, ÷òî âñå ëèöåíçèè äîñòóïíû

            var rlNode = licSettings.GetNode(attributeName) as IModelKeyString;
            return rlNode == null ? null : rlNode.Value;
        }
        private static LicensingSettings LicensingSettings()
        {
            var licSettings =
                // ReSharper disable once SuspiciousTypeConversion.Global
                ((SampleSettings)
                    (SettingsAccessor.Instance.ObjectSpace.AppSettings().Root())) ;

            return licSettings?.Licensing;
        }

        private static SampleSettings Settings()
        {
            if (SettingsAccessor.Instance == null) return null;
            if (!SettingsAccessor.Instance.Active) return null;

            var licSettings =
                // ReSharper disable once SuspiciousTypeConversion.Global
                ((SampleSettings)
                    (SettingsAccessor.Instance.ObjectSpace.AppSettings().Root()))
                    ;

            return licSettings;
        }
        

        private void Application_CreateCustomTemplate(object sender, CreateCustomTemplateEventArgs e)
        {
            if (e.Context == TemplateContext.ApplicationWindow)
            {
                e.Template = new Templates.ArmsMainFormTemplate();
            }
        }

        private void application_DatabaseVersionMismatch(object sender, DatabaseVersionMismatchEventArgs e)
        {
            e.Updater.Update();
            e.Handled = true;
        }

        private void application_CustomizeLanguagesList(object sender, CustomizeLanguagesListEventArgs e)
        {
            string userLanguageName = Thread.CurrentThread.CurrentUICulture.Name;
            if (userLanguageName != "en-US" && e.Languages.IndexOf(userLanguageName) == -1)
            {
                e.Languages.Add(userLanguageName);
            }
        }

        public IDataServerSecurity DataServerSecurity
        {
            get { return securityStrategyComplex; ; }
        }

        public override IEnumerable<ModuleUpdater> GetModuleUpdaters(IObjectSpace objectSpace, Version versionFromDB)
        {
            ModuleUpdater updater = new DatabaseUpdate.Updater(objectSpace, versionFromDB);
            return new ModuleUpdater[] { updater };
        }
        
        public override void CustomizeTypesInfo(ITypesInfo typesInfo)
        {
            base.CustomizeTypesInfo(typesInfo);
            CalculatedPersistentAliasHelper.CustomizeTypesInfo(typesInfo);
        }
        public override ICollection<Type> GetXafResourceLocalizerTypes()
        {
            return new Type[] { typeof(ModuleLocalizer) };
        }
    }
}
